"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _delay = _interopRequireDefault(require("delay"));

var _Logger = _interopRequireDefault(require("../Logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = _Logger.default.child({
  namespace: 'createHttpTerminator'
});

const configurationDefaults = {
  httpResponseTimeout: 1000
};

const createHttpTerminator = configurationInput => {
  const configuration = { ...configurationDefaults,
    ...configurationInput
  };
  const server = configuration.server;
  const sockets = new Set();
  server.on('connection', socket => {
    sockets.add(socket);
    server.once('close', () => {
      sockets.delete(socket);
    });
  });
  server.on('request', (incomingMessage, outgoingMessage) => {
    if (!outgoingMessage.headersSent) {
      outgoingMessage.setHeader('connection', 'close');
    }
  });
  let terminating;
  /**
   * Evaluate whether additional steps are required to destroy the socket.
   *
   * @see https://github.com/nodejs/node/blob/57bd715d527aba8dae56b975056961b0e429e91e/lib/_http_client.js#L363-L413
   */

  const destroySocket = socket => {
    socket.destroy();
    sockets.delete(socket);
  };

  const terminate = async () => {
    if (terminating) {
      log.warn('already terminating HTTP server');
      return terminating;
    }

    terminating = new Promise((resolve, reject) => {
      server.close(error => {
        if (error) {
          reject(error);
        } else {
          resolve();
        }
      });
    });

    for (const socket of sockets) {
      const serverResponse = socket._httpMessage;

      if (serverResponse) {
        if (!serverResponse.headersSent) {
          serverResponse.setHeader('connection', 'close');
        }

        continue;
      }

      destroySocket(socket);
    }

    if (sockets.size) {
      await (0, _delay.default)(configuration.httpResponseTimeout);

      for (const socket of sockets) {
        destroySocket(socket);
      }
    }

    return terminating;
  };

  return {
    terminate
  };
};

var _default = createHttpTerminator;
exports.default = _default;
//# sourceMappingURL=createHttpTerminator.js.map